/*
 * Project name:
     Thumbstick click - Thumbstick Click demo
 * Copyright:
     (c) mikroElektronika, 2015.
 * Revision History:
     20140723:
       - Initial release (DR & TP);
 * Description:
     This is a simple project which demonstrates the use of Thumbstick click board.
     Input device for 360 degree directional movement. USART Terminal shows direction.
     The joystick unit receives directional input in the x and y axes by way of two potentiometers connected to a stick. 
     The mechanical pushbutton sends an interrupt signal to the microcontroller. 
     Thumbstick click communicates with the target board through mikroBUS SPI (CS, SCK, MISO and MOSI) lines. 
     The board is designed to use either a 3.3V or a 5V power supply.
 * Test configuration:
     MCU:             PIC18F45K22
                      http://ww1.microchip.com/downloads/en/DeviceDoc/41412F.pdf
     Dev.Board:       EasyPIC7
                      http://www.mikroe.com/easypic/
     Oscillator:      HS-PLL 32.0000 MHz, 8.0000 MHz Crystal
     ext. modules:    Thumbstick click
                      http://www.mikroe.com/click/thumbstick/
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/mikroc/pic/
 * NOTES:
     - Set J3 and J4 jumpers on EasyPIC7 board in upper position for RS232 or lower position for USB UART.
     - Turn on SW1.1 and SW2.1 on EasyPIC7 board.
 */

// Thumbstick click module connections
sbit Chip_Select           at RE0_bit;
sbit Test                  at RD0_bit;
sbit Button                at RB0_bit;
sbit Chip_Select_Direction at TRISE0_bit;
sbit Test_Direction        at TRISD0_bit;
sbit Button_Direction      at TRISB0_bit;

unsigned int left_right, for_rev, old_left_right, old_for_rev;
unsigned int middle_left_right, middle_for_rev;


void interrupt() {
  if(INT0IF_bit) {
    INT0IF_bit = 0;
    Test = 1;                   // test
  }
}

// Get ADC values
unsigned int GetADC(unsigned short channel) {    // Returns 0..4095
  unsigned int tmp;
  Chip_Select = 0;                               // Select MCP3204
  SPI1_Write(0x06);                              // SPI communication using 8-bit segments
  channel = channel << 6;                        // Bits 7 & 6 define ADC input
  tmp = SPI1_Read(channel) & 0x0F;               // Get first 8 bits of ADC value
  tmp = tmp << 8;                                // Shift ADC value by 8
  tmp = tmp | SPI1_Read(0);                      // Get remaining 4 bits of ADC value
                                                 // and form 12-bit ADC value
  Chip_Select= 1;                                // Deselect MCP3204
  return tmp;                                    // Returns 12-bit ADC value
}

void Display_Button() {
if (Button  == 0) {
         UART1_Write_Text("  [Button] Pressed");
       }
         UART1_Write(13);
     if (Test && Button) {
       Test = 0;
     }
}

void DisplayValues() {
  int tmp;
  if(old_left_right != left_right) {
    tmp = middle_left_right - left_right;
    if(tmp < 0) {                                //move right
      UART1_Write_Text("[Right]   ");
    }else{                                       //move left
      UART1_Write_Text("[Left]    ");
    }
    Display_Button();
    UART1_Write(13);
    old_left_right = left_right;
  }

  if(old_for_rev != for_rev) {
    tmp = middle_for_rev - for_rev;
    if(tmp < 0) {                               //move reverse
      UART1_Write_Text("[Reverse] ");
    }else{                                      //move forward
      UART1_Write_Text("[Forward] ");
    }
    Display_Button();
	UART1_Write(13);
    old_for_rev = for_rev;
  }
}

void main() {
  // Configure AN pins as digital
  ANSELB = 0;
  ANSELC = 0;
  ANSELD = 0;
  ANSELE = 0;
  
   // Set Chip Select pin as Output
  Chip_Select_Direction = 0;
  Test_Direction = 0;
  Button_Direction = 1;

  UART1_Init(9600);
  
  SPI1_Init();
  Delay_ms(500);

  //Set middle values
  middle_for_rev = GetADC(0);
  middle_left_right = GetADC(1);

  INT0IE_bit  = 1;            // Enable interrupts on RB0
  INTEDG0_bit = 1;            // Interrupt on rising edge
  GIE_bit     = 1;
  PEIE_bit    = 1;
  INT0IE_bit = 1;

  while(1) {
     for_rev = GetADC(0);
     left_right = GetADC(1);
     DisplayValues();
     delay_ms(500);
  }
}